@@ -18,7 +18,7 @@ lav_constraints_parse <- function(partable = NULL, constraints = NULL,
         theta <- rep(0, length(partable$lhs))
     }
 
-    # number of free parameters
+    # number of free (but possibliy constrained) parameters
     npar <- length(theta)
 
     # parse the constraints
@@ -42,6 +42,12 @@ lav_constraints_parse <- function(partable = NULL, constraints = NULL,
         }
     }
 
+    # simple equality constraints?
+    ceq.simple <- FALSE
+    if(!is.null(partable$unco)) {
+        ceq.simple <- TRUE
+    }
+
     # variable definitions
     def.function <- lav_partable_constraints_def(partable, con = LIST,
                                                  debug = debug)
@@ -116,24 +122,26 @@ lav_constraints_parse <- function(partable = NULL, constraints = NULL,
     ceq.only.flag      <- ceq.flag && !cin.flag
     cin.only.flag      <- cin.flag && !ceq.flag
 
-    ceq.linear.only.flag <- (  ceq.linear.flag    && 
-                              !ceq.nonlinear.flag && 
+    ceq.linear.only.flag <- (  ceq.linear.flag    &&
+                              !ceq.nonlinear.flag &&
                               !cin.flag )
 
+    ceq.simple.only    <- ceq.simple && !ceq.flag && !cin.flag
+
     # additional info if ceq.linear.flag
     if(ceq.linear.flag) {
         ## NEW: 18 nov 2014: handle general *linear* constraints
         ##
         ## see Nocedal & Wright (2006) 15.3
-        ## - from x to x.red: 
+        ## - from x to x.red:
         ##       x.red <- MASS::ginv(Q2) %*% (x - Q1 %*% solve(t(R)) %*% b)
         ##   or
-        ##       x.red <- as.numeric((x - b %*% qr.coef(QR,diag(npar))) %*% Q2) 
-        ## 
+        ##       x.red <- as.numeric((x - b %*% qr.coef(QR,diag(npar))) %*% Q2)
+        ##
         ## - from x.red to x
-        ##       x <- as.numeric(Q1 %*% solve(t(R)) %*% b + Q2 %*% x.red) 
+        ##       x <- as.numeric(Q1 %*% solve(t(R)) %*% b + Q2 %*% x.red)
         ##   or
-        ##       x <- as.numeric(b %*% qr.coef(QR, diag(npar))) + 
+        ##       x <- as.numeric(b %*% qr.coef(QR, diag(npar))) +
         ##                       as.numeric(Q2 %*% x.red)
         ##
         ## we write eq.constraints.K = Q2
@@ -166,6 +174,22 @@ lav_constraints_parse <- function(partable = NULL, constraints = NULL,
         ceq.rhs.NULL <- numeric(0L)
     }
 
+    # if simple equalities only, create 'K' matrix
+    ceq.simple.K <- matrix(0,0,0)
+    if(ceq.simple.only) {
+        n.unco <- max(partable$unco)
+        n.free <- max(partable$free)
+        ceq.simple.K <- matrix(0, nrow = n.unco, ncol = n.free)
+        #####
+        #####     FIXME !
+        #####
+        idx.free <- partable$free[ partable$free > 0 ]
+        for(k in 1:n.unco) {
+            c <- idx.free[k]
+            ceq.simple.K[k, c] <- 1
+        }
+    }
+
     # dummy jacobian 'function'
     ceq.jacobian <- function() NULL
     cin.jacobian <- function() NULL
@@ -186,6 +210,8 @@ lav_constraints_parse <- function(partable = NULL, constraints = NULL,
                 ceq.linear.only.flag = ceq.linear.only.flag,
                 ceq.JAC.NULL         = ceq.JAC.NULL,
                 ceq.rhs.NULL         = ceq.rhs.NULL,
+                ceq.simple.only      = ceq.simple.only,
+                ceq.simple.K         = ceq.simple.K,
 
                 cin.function         = cin.function,
                 cin.JAC              = cin.JAC,
@@ -202,10 +228,10 @@ lav_constraints_parse <- function(partable = NULL, constraints = NULL,
     OUT
 }
 
-lav_constraints_linear_idx <- function(func = NULL, npar = NULL) { 
+lav_constraints_linear_idx <- function(func = NULL, npar = NULL) {
 
     if(is.null(func) || is.null(body(func))) return(integer(0L))
-    
+
     # seed 1: rnorm
     A0 <- lav_func_jacobian_complex(func = func, x = rnorm(npar))
 
@@ -217,8 +243,8 @@ lav_constraints_linear_idx <- function(func = NULL, npar = NULL) {
     which(linear)
 }
 
-lav_constraints_nonlinear_idx <- function(func = NULL, npar = NULL) {   
-  
+lav_constraints_nonlinear_idx <- function(func = NULL, npar = NULL) {
+
     if(is.null(func) || is.null(body(func))) return(integer(0L))
 
     # seed 1: rnorm
@@ -242,7 +268,7 @@ lav_constraints_check_linear <- function(model) {
          A.ceq <- t(lav_func_jacobian_complex(func=model@ceq.function, x=rnorm(model@nx.free)))
      if(!is.null(body(model@cin.function)))
          A.cin <- t(lav_func_jacobian_complex(func=model@cin.function, x=rnorm(model@nx.free)))
-     A0 <- cbind(A.ceq, A.cin)    
+     A0 <- cbind(A.ceq, A.cin)
 
      # seed 2: rnorm
      A.ceq <- A.cin <- matrix(0, model@nx.free, 0)
@@ -290,9 +316,44 @@ lav_constraints_R2K <- function(lavmodel = NULL, R = NULL) {
         K[idx2, idx1] <- 1
     }
 
-    # remove redundant columns 
+    # remove redundant columns
     neg.idx <- which(colSums(R) < 0)
     K <- K[,-neg.idx]
 
     K
 }
+
+lav_constraints_lambda_pre <- function(lavobject = NULL, method = "Don") {
+
+    # compute factor 'pre' so that pre %*% g = lambda
+    method <- tolower(method)
+
+    R <- lavobject@Model@con.jac[,]
+    if(is.null(R) || length(R) == 0L) {
+        return( numeric(0L) )
+    }
+
+    INFO <- lavTech(lavobject, "information.first.order")
+    npar <- nrow(INFO)
+
+    # Don 1985
+    if(method == "don") {
+        R.plus <- MASS::ginv(R)
+
+        # construct augmented matrix
+        Z <- rbind( cbind(INFO, t(R)),
+                    cbind(R, matrix(0,nrow=nrow(R), ncol=nrow(R))) )
+        Z.plus <- MASS::ginv(Z)
+        P.star <- Z.plus[1:npar, 1:npar]
+        PRE <- t(R.plus) %*% (diag(npar) - INFO %*% P.star)
+
+    # Bentler EQS manual
+    } else if(method == "bentler") {
+        INFO.inv <- solve(INFO)
+        PRE <- solve( R %*% INFO.inv %*% t(R) ) %*% R %*% INFO.inv
+    }
+
+    PRE
+}
+
+
