@@ -12,6 +12,7 @@ lavCor <- function(object,
                    group      = NULL,
                    missing    = "listwise",
                    ov.names.x = NULL,
+                   sampling.weights = NULL,
                    # lavaan options
                    se         = "none",
                    test       = "none",
@@ -20,7 +21,7 @@ lavCor <- function(object,
                    # other options (for lavaan)
                    ...,
                    cor.smooth = FALSE,
-                   cor.smooth.tol = 1e-06,
+                   cor.smooth.tol = 1e-04, # was 1e-06 in <0.6-14
                    output = "cor") {
 
     # shortcut if object = lavaan object
@@ -44,18 +45,58 @@ lavCor <- function(object,
         }
     }
 
+    # extract sampling.weights.normalization from dots (for lavData() call)
+    dots <- list(...)
+    sampling.weights.normalization <- "total"
+    if (!is.null(dots$sampling.weights.normalization)) {
+        sampling.weights.normalization <- dots$sampling.weights.normalization
+    }
+
+
     # check object class
     if(inherits(object, "lavData")) {
         lav.data <- object
-    } else if(inherits(object, "data.frame")) {
+    } else if(inherits(object, "data.frame") ||
+              inherits(object, "matrix")) {
+        object <- as.data.frame(object)
         NAMES <- names(object)
         if(!is.null(group)) {
             NAMES <- NAMES[- match(group, NAMES)]
         }
+        if(!is.null(sampling.weights)) {
+            NAMES <- NAMES[- match(sampling.weights, NAMES)]
+        }
+        if(is.logical(ordered)) {
+            ordered.flag <- ordered
+            if(ordered.flag) {
+                ordered <- NAMES
+                if(length(ov.names.x) > 0L) {
+                    ordered <- ordered[- which(ordered %in% ov.names.x) ]
+                }
+            } else {
+                ordered <- character(0L)
+            }
+        } else if(is.null(ordered)) {
+            ordered <- character(0L)
+        } else if(!is.character(ordered)) {
+            stop("lavaan ERROR: ordered argument must be a character vector")
+        } else if(length(ordered) == 1L && nchar(ordered) == 0L) {
+            ordered <- character(0L)
+        } else {
+            # check if all names in "ordered" occur in the dataset?
+            missing.idx <- which(!ordered %in% NAMES)
+            if(length(missing.idx) > 0L) { # FIXme: warn = FALSE has no eff
+                warning("lavaan WARNING: ordered variable(s): ",
+                     paste(ordered[missing.idx], collapse = " "),
+                     "\n  could not be found in the data and will be ignored")
+            }
+        }
         lav.data <- lavData(data = object, group = group,
                             ov.names = NAMES, ordered = ordered,
+                            sampling.weights = sampling.weights,
                             ov.names.x = ov.names.x,
-                            lavoptions = list(missing = missing))
+                            lavoptions = list(missing = missing,
+                                              sampling.weights.normalization = sampling.weights.normalization))
     } else {
         stop("lavaan ERROR: lavCor can not handle objects of class ",
              paste(class(object), collapse= " "))
@@ -71,8 +112,7 @@ lavCor <- function(object,
         }
     }
 
-    # extract partable options from dots
-    dots <- list(...)
+    # extract more partable options from dots
     meanstructure <- FALSE
     fixed.x       <- FALSE
     mimic         <- "lavaan"
@@ -93,7 +133,6 @@ lavCor <- function(object,
         conditional.x <- dots$conditional.x
     }
 
-
     # override, only for backwards compatibility (eg moments() in JWileymisc)
     #if(missing %in% c("ml", "fiml")) {
     #    meanstructure = TRUE
@@ -106,6 +145,7 @@ lavCor <- function(object,
                                   lavoptions    = list(meanstructure = meanstructure,
                                                        fixed.x = fixed.x,
                                                        conditional.x = conditional.x,
+                                                       # sampling.weights.normalization = sampling.weights.normalization,
                                                        group.w.free = FALSE,
                                                        missing = missing,
                                                        correlation = categorical,
