@@ -21,6 +21,7 @@ function(object, fit.measures = "all", baseline.model = NULL,
                         rmsea.ci.level    = 0.90,
                         rmsea.close.h0    = 0.05,
                         rmsea.notclose.h0 = 0.08,
+                        robust            = TRUE,
                         cat.check.pd      = TRUE),
          output = "vector", ...) {
     # note: the ... is not used by lavaan
@@ -36,6 +37,7 @@ function(object, fit.measures = "all", baseline.model = NULL,
                         rmsea.ci.level    = 0.90,
                         rmsea.close.h0    = 0.05,
                         rmsea.notclose.h0 = 0.08,
+                        robust            = TRUE,
                         cat.check.pd      = TRUE),
          output = "vector",  ...) {
     # note: the ... is not used by lavaan
@@ -53,6 +55,7 @@ fitMeasures.efaList <- fitmeasures.efaList <- function(object,
                    rmsea.ci.level    = 0.90,
                    rmsea.close.h0    = 0.05,
                    rmsea.notclose.h0 = 0.08,
+                   robust            = TRUE,
                    cat.check.pd      = TRUE),
     output = "list", ...) {
 
@@ -97,6 +100,7 @@ lav_fit_measures <- function(object, fit.measures = "all",
                                             rmsea.ci.level    = 0.90,
                                             rmsea.close.h0    = 0.05,
                                             rmsea.notclose.h0 = 0.08,
+                                            robust            = TRUE,
                                             cat.check.pd      = TRUE),
                              output = "vector") {
 
@@ -106,6 +110,7 @@ lav_fit_measures <- function(object, fit.measures = "all",
                             rmsea.ci.level    = 0.90,
                             rmsea.close.h0    = 0.05,
                             rmsea.notclose.h0 = 0.08,
+                            robust            = TRUE,
                             cat.check.pd      = TRUE)
     if(!missing(fm.args)) {
         fm.args <- modifyList(default.fm.args, fm.args)
@@ -214,7 +219,8 @@ lav_fit_measures <- function(object, fit.measures = "all",
 
     # options
     categorical.flag <- object@Model@categorical
-    fiml.flag        <- object@Options$missing %in% c("ml", "ml.x")
+    fiml.flag        <- ( fm.args$robust &&
+                          object@Options$missing %in% c("ml", "ml.x") )
     estimator        <- object@Options$estimator
 
     # basic ingredients
@@ -251,7 +257,7 @@ lav_fit_measures <- function(object, fit.measures = "all",
     if(scaled.flag) {
         fit.cfi.tli <- c(fit.cfi.tli, "cfi.scaled", "tli.scaled")
     }
-    if(scaled.flag || categorical.flag || fiml.flag) {
+    if(fm.args$robust && (scaled.flag || categorical.flag || fiml.flag)) {
         fit.cfi.tli <- c(fit.cfi.tli, "cfi.robust", "tli.robust")
     }
 
@@ -261,7 +267,7 @@ lav_fit_measures <- function(object, fit.measures = "all",
         fit.cfi.other <- c(fit.cfi.other, "nnfi.scaled", "rfi.scaled",
                        "nfi.scaled", "pnfi.scaled", "ifi.scaled", "rni.scaled")
     }
-    if(scaled.flag || categorical.flag || fiml.flag) {
+    if(fm.args$robust && (scaled.flag || categorical.flag || fiml.flag)) {
         fit.cfi.other <- c(fit.cfi.other, "nnfi.robust", "rni.robust")
     }
     fit.cfi <- c(fit.baseline, fit.cfi.tli, fit.cfi.other)
@@ -288,7 +294,7 @@ lav_fit_measures <- function(object, fit.measures = "all",
                        "rmsea.ci.upper.scaled", "rmsea.pvalue.scaled",
                        "rmsea.notclose.pvalue.scaled")
     }
-    if(scaled.flag || categorical.flag || fiml.flag) {
+    if(fm.args$robust && (scaled.flag || categorical.flag || fiml.flag)) {
         fit.rmsea <- c(fit.rmsea, "rmsea.robust", "rmsea.ci.lower.robust",
                        "rmsea.ci.upper.robust", "rmsea.pvalue.robust",
                        "rmsea.notclose.pvalue.robust")
@@ -404,6 +410,7 @@ lav_fit_measures <- function(object, fit.measures = "all",
                                            baseline.model = baseline.model,
                                            standard.test  = standard.test,
                                            scaled.test    = scaled.test,
+                                           robust         = fm.args$robust,
                                            cat.check.pd   = fm.args$cat.check.pd))
     }
 
@@ -454,6 +461,7 @@ lav_fit_measures <- function(object, fit.measures = "all",
                                      ci.level          = rmsea.ci.level,
                                      close.h0          = rmsea.close.h0,
                                      notclose.h0       = rmsea.notclose.h0,
+                                     robust            = fm.args$robust,
                                      cat.check.pd      = fm.args$cat.check.pd))
     }
 
