@@ -31,7 +31,7 @@ lav_model_h1_information <- function(lavobject      = NULL,
         if(.hasSlot(lavobject, "h1")) {
             lavh1      <- lavobject@h1
         } else {
-            lavh1      <- lav_h1_logl(lavdata = lavobject@Data,
+            lavh1      <- lav_h1_implied_logl(lavdata = lavobject@Data,
                                       lavsamplestats = lavobject@SampleStats,
                                       lavoptions = lavobject@Options)
         }
@@ -39,6 +39,16 @@ lav_model_h1_information <- function(lavobject      = NULL,
         lavoptions     <- lavobject@Options
     }
 
+    # sanity check
+    if(length(lavh1) == 0L) {
+        lavh1 <- lav_h1_implied_logl(lavdata = lavdata,
+                                     lavsamplestats = lavsamplestats,
+                                     lavoptions = lavoptions)
+    }
+    if(length(lavimplied) == 0L) {
+        lavimplied <- lav_model_implied(lavmodel = lavmodel)
+    }
+
     # information
     information <- lavoptions$information[1] # ALWAYS take the first one
                                              # the caller must control it
@@ -84,7 +94,7 @@ lav_model_h1_information_expected <- function(lavobject      = NULL,
         if(.hasSlot(lavobject, "h1")) {
             lavh1      <- lavobject@h1
         } else {
-            lavh1      <- lav_h1_logl(lavdata = lavobject@Data,
+            lavh1      <- lav_h1_implied_logl(lavdata = lavobject@Data,
                                       lavsamplestats = lavobject@SampleStats,
                                       lavoptions = lavobject@Options)
         }
@@ -92,6 +102,16 @@ lav_model_h1_information_expected <- function(lavobject      = NULL,
         lavoptions     <- lavobject@Options
     }
 
+    # sanity check
+    if(length(lavh1) == 0L) {
+        lavh1 <- lav_h1_implied_logl(lavdata = lavdata,
+                                     lavsamplestats = lavsamplestats,
+                                     lavoptions = lavoptions)
+    }
+    if(length(lavimplied) == 0L) {
+        lavimplied <- lav_model_implied(lavmodel = lavmodel)
+    }
+
     estimator <- lavmodel@estimator
 
     # structured of unstructured? (since 0.5-23)
@@ -139,7 +159,7 @@ lav_model_h1_information_expected <- function(lavobject      = NULL,
     }
 
     # 3a. ML single level
-    else if( lavmodel@estimator %in% c("ML", "NTRLS", "DLS")
+    else if( lavmodel@estimator %in% c("ML", "NTRLS", "DLS", "catML")
              && lavdata@nlevels == 1L ) {
         A1 <- vector("list", length=lavsamplestats@ngroups)
 
@@ -229,18 +249,24 @@ lav_model_h1_information_expected <- function(lavobject      = NULL,
                         MEAN <- lavsamplestats@mean[[g]]
                     }
 
+                    correlation.flag <- FALSE
+                    if(.hasSlot(lavmodel, "correlation")) {
+                        correlation.flag <- lavmodel@correlation
+                    }
                     if(structured) {
                         A1[[g]] <- lav_mvnorm_information_expected(
                                   Sigma         = lavimplied$cov[[g]],
                                   #wt = WT, # not needed
                                   x.idx         = lavsamplestats@x.idx[[g]],
-                                  meanstructure = lavmodel@meanstructure)
+                                  meanstructure = lavmodel@meanstructure,
+                                  correlation   = correlation.flag)
                     } else {
                         A1[[g]] <- lav_mvnorm_h1_information_expected(
                                   sample.cov.inv = lavsamplestats@icov[[g]],
                                   #wt = WT, not needed
                                   x.idx          = lavsamplestats@x.idx[[g]],
-                                  meanstructure  = lavmodel@meanstructure)
+                                  meanstructure  = lavmodel@meanstructure,
+                                  correlation    = correlation.flag)
                     }
                 } # conditional.x
             } # missing
@@ -311,7 +337,7 @@ lav_model_h1_information_observed <- function(lavobject      = NULL,
         if(.hasSlot(lavobject, "h1")) {
             lavh1      <- lavobject@h1
         } else {
-            lavh1      <- lav_h1_logl(lavdata = lavobject@Data,
+            lavh1      <- lav_h1_implied_logl(lavdata = lavobject@Data,
                                       lavsamplestats = lavobject@SampleStats,
                                       lavoptions = lavobject@Options)
         }
@@ -319,6 +345,16 @@ lav_model_h1_information_observed <- function(lavobject      = NULL,
         lavoptions     <- lavobject@Options
     }
 
+    # sanity check
+    if(length(lavh1) == 0L) {
+        lavh1 <- lav_h1_implied_logl(lavdata = lavdata,
+                                     lavsamplestats = lavsamplestats,
+                                     lavoptions = lavoptions)
+    }
+    if(length(lavimplied) == 0L) {
+        lavimplied <- lav_model_implied(lavmodel = lavmodel)
+    }
+
     estimator <- lavmodel@estimator
 
     # structured?
@@ -539,7 +575,7 @@ lav_model_h1_information_firstorder <- function(lavobject      = NULL,
         if(.hasSlot(lavobject, "h1")) {
             lavh1      <- lavobject@h1
         } else {
-            lavh1      <- lav_h1_logl(lavdata = lavobject@Data,
+            lavh1      <- lav_h1_implied_logl(lavdata = lavobject@Data,
                                       lavsamplestats = lavobject@SampleStats,
                                       lavoptions = lavobject@Options)
         }
@@ -547,6 +583,16 @@ lav_model_h1_information_firstorder <- function(lavobject      = NULL,
         lavoptions     <- lavobject@Options
     }
 
+    # sanity check
+    if(length(lavh1) == 0L) {
+        lavh1 <- lav_h1_implied_logl(lavdata = lavdata,
+                                     lavsamplestats = lavsamplestats,
+                                     lavoptions = lavoptions)
+    }
+    if(length(lavimplied) == 0L) {
+        lavimplied <- lav_model_implied(lavmodel = lavmodel)
+    }
+
     estimator <- lavmodel@estimator
     if(!estimator %in% c("ML", "PML")) {
         stop("lavaan ERROR: information = \"first.order\" not available for estimator ", sQuote(estimator))
@@ -799,7 +845,7 @@ lav_model_h1_acov <- function(lavobject      = NULL,
         if(.hasSlot(lavobject, "h1")) {
             lavh1      <- lavobject@h1
         } else {
-            lavh1      <- lav_h1_logl(lavdata = lavobject@Data,
+            lavh1      <- lav_h1_implied_logl(lavdata = lavobject@Data,
                                       lavsamplestats = lavobject@SampleStats,
                                       lavoptions = lavobject@Options)
         }
@@ -807,6 +853,16 @@ lav_model_h1_acov <- function(lavobject      = NULL,
         lavoptions     <- lavobject@Options
     }
 
+    # sanity check
+    if(length(lavh1) == 0L) {
+        lavh1 <- lav_h1_implied_logl(lavdata = lavdata,
+                                     lavsamplestats = lavsamplestats,
+                                     lavoptions = lavoptions)
+    }
+    if(length(lavimplied) == 0L) {
+        lavimplied <- lav_model_implied(lavmodel = lavmodel)
+    }
+
     # override
     if(!is.null(meanstructure)) {
         lavoptions$meanstructure <- meanstructure
