@@ -35,7 +35,7 @@ lav_object_independence <- function(object         = NULL,
         if(.hasSlot(object, "h1"))  {
             lavh1 <- object@h1
         } else {
-            lavh1 <- lav_h1_logl(lavdata = object@Data,
+            lavh1 <- lav_h1_implied_logl(lavdata = object@Data,
                                  lavsamplestats = object@SampleStats,
                                  lavoptions = object@Options)
         }
@@ -192,7 +192,7 @@ lav_object_unrestricted <- function(object, se = FALSE, verbose = FALSE,
     if(.hasSlot(object, "h1"))  {
         lavh1 <- object@h1
     } else {
-        lavh1 <- lav_h1_logl(lavdata = object@Data,
+        lavh1 <- lav_h1_implied_logl(lavdata = object@Data,
                              lavsamplestats = object@SampleStats,
                              lavoptions = object@Options)
     }
@@ -362,7 +362,7 @@ lav_object_extended <- function(object, add = NULL,
         object@Model@estimator <- object@Options$estimator
         object@Model@estimator.args <- list()
 
-        lavh1 <- lav_h1_logl(lavdata = object@Data,
+        lavh1 <- lav_h1_implied_logl(lavdata = object@Data,
                              lavsamplestats = object@SampleStats,
                              lavoptions = object@Options)
     }
@@ -418,10 +418,18 @@ lav_object_catml <- function(lavobject = NULL) {
     for(g in seq_len(lavdata@ngroups)) {
         lavsamplestats@WLS.V[[g]] <- NULL
         lavsamplestats@WLS.VD[[g]] <- NULL
-        COV <- cov2cor(lav_matrix_symmetric_force_pd(lavsamplestats@cov[[g]],
-                                                     tol = 1e-06))
-        lavsamplestats@cov[[g]] <- COV
-        lavsamplestats@var[[g]] <- diag(COV)
+        COR <- lavsamplestats@cov[[g]]
+        # check if COV is pd or not
+        ev <- eigen(COR, symmetric = TRUE, only.values = TRUE)$values
+        if(any(ev < .Machine$double.eps^(1/2))) {
+            # not PD!
+            COV <- cov2cor(lav_matrix_symmetric_force_pd(COR, tol = 1e-04))
+            lavsamplestats@cov[[g]] <- COV
+            lavsamplestats@var[[g]] <- diag(COV)
+            refit <- TRUE
+        } else {
+            COV <- COR
+        }
 
         out <- lav_samplestats_icov(COV = COV, ridge = 1e-05,
                                     x.idx = lavsamplestats@x.idx[[g]],
