@@ -23,6 +23,9 @@ lav_start <- function(start.method    = "default",
     # categorical?
     categorical <- any(lavpartable$op == "|")
 
+    # correlation structure?
+    correlation <- any(lavpartable$op == "~*~")
+
     # conditional.x?
     conditional.x <- any(lavpartable$exo == 1L &
                          lavpartable$op %in% c("~", "<~"))
@@ -47,7 +50,7 @@ lav_start <- function(start.method    = "default",
     } else if(mimic == "Mplus") {
         start.initial <- "mplus"
     } else {
-        # FIXME: use LISREL/EQS/AMOS/.... schems
+        # FIXME: use LISREL/EQS/AMOS/.... schemes
         start.initial <- "lavaan"
     }
 
@@ -59,7 +62,11 @@ lav_start <- function(start.method    = "default",
             # nothing to do
         } else if(start.method == "simple") {
             start <- numeric( length(lavpartable$ustart) )
-            start[ which(lavpartable$op == "=~") ] <- 1.0
+            if(categorical || correlation) {
+                start[ which(lavpartable$op == "=~") ] <- 0.7
+            } else {
+                start[ which(lavpartable$op == "=~") ] <- 1.0
+            }
             start[ which(lavpartable$op == "~*~") ] <- 1.0
             ov.names.ord <- vnames(lavpartable, "ov.ord")
             var.idx <- which(lavpartable$op == "~~" &
@@ -114,7 +121,7 @@ lav_start <- function(start.method    = "default",
     start <- numeric( length(lavpartable$ustart) )
 
     # 1. =~ factor loadings:
-    if(categorical) {
+    if(categorical || correlation) {
         # if std.lv=TRUE, 0.8 is too large
         start[ which(lavpartable$op == "=~") ] <- 0.7
     } else {
@@ -262,7 +269,11 @@ lav_start <- function(start.method    = "default",
 
                     # check for negative triad if nvar=3L (new in 0.6-8)
                     if(!is.null(fabin$neg.triad) && fabin$neg.triad) {
-                        tmp <- rep(1.0, length(tmp))
+                        if(std.lv) {
+                            tmp <- rep(0.7, length(tmp))
+                        } else {
+                            tmp <- rep(1.0, length(tmp))
+                        }
                     }
                     start[lambda.idx] <- tmp
 
@@ -409,6 +420,11 @@ lav_start <- function(start.method    = "default",
             }
         }
 
+        # TODo: if marker.int.zero = TRUE, set lv means to marker means,
+        #       and the non-marker means to
+        #       lavsamplestats@mean[[g]] - LAMBDA %*% ALPHA
+        #       where ALPHA = means of the markers
+
         # 4g) thresholds
         th.idx <- which(lavpartable$group == group.values[g] & lavpartable$op == "|")
         if(length(th.idx) > 0L) {
@@ -540,12 +556,25 @@ lav_start <- function(start.method    = "default",
                                      lavpartable$lhs == y.name &
                                      lavpartable$rhs == y.name)
                     res.val <- COV[y.idx, y.idx] - drop(crossprod(beta.i, S.xy))
-                    if(res.val > 0.01*COV[y.idx, y.idx] &&
-                       res.val < 0.99*COV[y.idx, y.idx]) {
+                    if(res.val > 0.001*COV[y.idx, y.idx] &&
+                       res.val < 0.999*COV[y.idx, y.idx]) {
                         start[res.idx] <- res.val
                     } else {
                         # do nothing (keep what we have)
                     }
+                    # intercept
+                    int.idx <- which(lavpartable$group == group.values[g] &
+                                     lavpartable$op == "~1" &
+                                     lavpartable$lhs == y.name)
+                    if(length(int.idx) > 0L) {
+                        MEAN <- lavsamplestats@mean[[g]]
+                        Ybar <- MEAN[y.idx]
+                        Xbar <- MEAN[x.idx]
+                        int.val <- Ybar - drop(crossprod(beta.i, Xbar))
+                        if(is.finite(int.val)) {
+                            start[int.idx] <- int.val
+                        }
+                    }
                 }
             }
         }
@@ -803,6 +832,13 @@ lav_start <- function(start.method    = "default",
     }
 
 
+    # override if the model syntax contains explicit starting values (free only)
+    #user.idx <- which(!is.na(lavpartable$ustart) &
+    #                  lavpartable$user != 7L) # new in 0.6-7, if rotation and
+    #                                          # and we change the order of lv's
+    user.idx <- which(!is.na(lavpartable$ustart) & lavpartable$free > 0L)
+    start[user.idx] <- lavpartable$ustart[user.idx]
+
     # override if a user list with starting values is provided
     # we only look at the 'est' column for now
     if(!is.null(start.user)) {
@@ -833,11 +869,8 @@ lav_start <- function(start.method    = "default",
         }
     }
 
-    # override if the model syntax contains explicit starting values
-    #user.idx <- which(!is.na(lavpartable$ustart) &
-    #                  lavpartable$user != 7L) # new in 0.6-7, if rotation and
-    #                                          # and we change the order of lv's
-    user.idx <- which(!is.na(lavpartable$ustart))
+    # override fixed values with ustart values
+    user.idx <- which(!is.na(lavpartable$ustart) & lavpartable$free == 0L)
     start[user.idx] <- lavpartable$ustart[user.idx]
 
     # final check: no NaN or other non-finite values
